import datetime

from google.protobuf import timestamp_pb2 as _timestamp_pb2
import yellowstone_fumarole_proto.solana_storage_pb2 as _solana_storage_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from collections.abc import Iterable as _Iterable, Mapping as _Mapping
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union
from yellowstone_fumarole_proto.solana_storage_pb2 import ConfirmedBlock as ConfirmedBlock
from yellowstone_fumarole_proto.solana_storage_pb2 import ConfirmedTransaction as ConfirmedTransaction
from yellowstone_fumarole_proto.solana_storage_pb2 import Transaction as Transaction
from yellowstone_fumarole_proto.solana_storage_pb2 import Message as Message
from yellowstone_fumarole_proto.solana_storage_pb2 import MessageHeader as MessageHeader
from yellowstone_fumarole_proto.solana_storage_pb2 import MessageAddressTableLookup as MessageAddressTableLookup
from yellowstone_fumarole_proto.solana_storage_pb2 import TransactionStatusMeta as TransactionStatusMeta
from yellowstone_fumarole_proto.solana_storage_pb2 import TransactionError as TransactionError
from yellowstone_fumarole_proto.solana_storage_pb2 import InnerInstructions as InnerInstructions
from yellowstone_fumarole_proto.solana_storage_pb2 import InnerInstruction as InnerInstruction
from yellowstone_fumarole_proto.solana_storage_pb2 import CompiledInstruction as CompiledInstruction
from yellowstone_fumarole_proto.solana_storage_pb2 import TokenBalance as TokenBalance
from yellowstone_fumarole_proto.solana_storage_pb2 import UiTokenAmount as UiTokenAmount
from yellowstone_fumarole_proto.solana_storage_pb2 import ReturnData as ReturnData
from yellowstone_fumarole_proto.solana_storage_pb2 import Reward as Reward
from yellowstone_fumarole_proto.solana_storage_pb2 import Rewards as Rewards
from yellowstone_fumarole_proto.solana_storage_pb2 import UnixTimestamp as UnixTimestamp
from yellowstone_fumarole_proto.solana_storage_pb2 import BlockHeight as BlockHeight
from yellowstone_fumarole_proto.solana_storage_pb2 import NumPartitions as NumPartitions
from yellowstone_fumarole_proto.solana_storage_pb2 import RewardType as RewardType

DESCRIPTOR: _descriptor.FileDescriptor
Unspecified: _solana_storage_pb2.RewardType
Fee: _solana_storage_pb2.RewardType
Rent: _solana_storage_pb2.RewardType
Staking: _solana_storage_pb2.RewardType
Voting: _solana_storage_pb2.RewardType

class CommitmentLevel(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    PROCESSED: _ClassVar[CommitmentLevel]
    CONFIRMED: _ClassVar[CommitmentLevel]
    FINALIZED: _ClassVar[CommitmentLevel]

class SlotStatus(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    SLOT_PROCESSED: _ClassVar[SlotStatus]
    SLOT_CONFIRMED: _ClassVar[SlotStatus]
    SLOT_FINALIZED: _ClassVar[SlotStatus]
    SLOT_FIRST_SHRED_RECEIVED: _ClassVar[SlotStatus]
    SLOT_COMPLETED: _ClassVar[SlotStatus]
    SLOT_CREATED_BANK: _ClassVar[SlotStatus]
    SLOT_DEAD: _ClassVar[SlotStatus]
PROCESSED: CommitmentLevel
CONFIRMED: CommitmentLevel
FINALIZED: CommitmentLevel
SLOT_PROCESSED: SlotStatus
SLOT_CONFIRMED: SlotStatus
SLOT_FINALIZED: SlotStatus
SLOT_FIRST_SHRED_RECEIVED: SlotStatus
SLOT_COMPLETED: SlotStatus
SLOT_CREATED_BANK: SlotStatus
SLOT_DEAD: SlotStatus

class SubscribeRequest(_message.Message):
    __slots__ = ("accounts", "slots", "transactions", "transactions_status", "blocks", "blocks_meta", "entry", "commitment", "accounts_data_slice", "ping", "from_slot")
    class AccountsEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: SubscribeRequestFilterAccounts
        def __init__(self, key: _Optional[str] = ..., value: _Optional[_Union[SubscribeRequestFilterAccounts, _Mapping]] = ...) -> None: ...
    class SlotsEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: SubscribeRequestFilterSlots
        def __init__(self, key: _Optional[str] = ..., value: _Optional[_Union[SubscribeRequestFilterSlots, _Mapping]] = ...) -> None: ...
    class TransactionsEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: SubscribeRequestFilterTransactions
        def __init__(self, key: _Optional[str] = ..., value: _Optional[_Union[SubscribeRequestFilterTransactions, _Mapping]] = ...) -> None: ...
    class TransactionsStatusEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: SubscribeRequestFilterTransactions
        def __init__(self, key: _Optional[str] = ..., value: _Optional[_Union[SubscribeRequestFilterTransactions, _Mapping]] = ...) -> None: ...
    class BlocksEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: SubscribeRequestFilterBlocks
        def __init__(self, key: _Optional[str] = ..., value: _Optional[_Union[SubscribeRequestFilterBlocks, _Mapping]] = ...) -> None: ...
    class BlocksMetaEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: SubscribeRequestFilterBlocksMeta
        def __init__(self, key: _Optional[str] = ..., value: _Optional[_Union[SubscribeRequestFilterBlocksMeta, _Mapping]] = ...) -> None: ...
    class EntryEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: SubscribeRequestFilterEntry
        def __init__(self, key: _Optional[str] = ..., value: _Optional[_Union[SubscribeRequestFilterEntry, _Mapping]] = ...) -> None: ...
    ACCOUNTS_FIELD_NUMBER: _ClassVar[int]
    SLOTS_FIELD_NUMBER: _ClassVar[int]
    TRANSACTIONS_FIELD_NUMBER: _ClassVar[int]
    TRANSACTIONS_STATUS_FIELD_NUMBER: _ClassVar[int]
    BLOCKS_FIELD_NUMBER: _ClassVar[int]
    BLOCKS_META_FIELD_NUMBER: _ClassVar[int]
    ENTRY_FIELD_NUMBER: _ClassVar[int]
    COMMITMENT_FIELD_NUMBER: _ClassVar[int]
    ACCOUNTS_DATA_SLICE_FIELD_NUMBER: _ClassVar[int]
    PING_FIELD_NUMBER: _ClassVar[int]
    FROM_SLOT_FIELD_NUMBER: _ClassVar[int]
    accounts: _containers.MessageMap[str, SubscribeRequestFilterAccounts]
    slots: _containers.MessageMap[str, SubscribeRequestFilterSlots]
    transactions: _containers.MessageMap[str, SubscribeRequestFilterTransactions]
    transactions_status: _containers.MessageMap[str, SubscribeRequestFilterTransactions]
    blocks: _containers.MessageMap[str, SubscribeRequestFilterBlocks]
    blocks_meta: _containers.MessageMap[str, SubscribeRequestFilterBlocksMeta]
    entry: _containers.MessageMap[str, SubscribeRequestFilterEntry]
    commitment: CommitmentLevel
    accounts_data_slice: _containers.RepeatedCompositeFieldContainer[SubscribeRequestAccountsDataSlice]
    ping: SubscribeRequestPing
    from_slot: int
    def __init__(self, accounts: _Optional[_Mapping[str, SubscribeRequestFilterAccounts]] = ..., slots: _Optional[_Mapping[str, SubscribeRequestFilterSlots]] = ..., transactions: _Optional[_Mapping[str, SubscribeRequestFilterTransactions]] = ..., transactions_status: _Optional[_Mapping[str, SubscribeRequestFilterTransactions]] = ..., blocks: _Optional[_Mapping[str, SubscribeRequestFilterBlocks]] = ..., blocks_meta: _Optional[_Mapping[str, SubscribeRequestFilterBlocksMeta]] = ..., entry: _Optional[_Mapping[str, SubscribeRequestFilterEntry]] = ..., commitment: _Optional[_Union[CommitmentLevel, str]] = ..., accounts_data_slice: _Optional[_Iterable[_Union[SubscribeRequestAccountsDataSlice, _Mapping]]] = ..., ping: _Optional[_Union[SubscribeRequestPing, _Mapping]] = ..., from_slot: _Optional[int] = ...) -> None: ...

class SubscribeRequestFilterAccounts(_message.Message):
    __slots__ = ("account", "owner", "filters", "nonempty_txn_signature")
    ACCOUNT_FIELD_NUMBER: _ClassVar[int]
    OWNER_FIELD_NUMBER: _ClassVar[int]
    FILTERS_FIELD_NUMBER: _ClassVar[int]
    NONEMPTY_TXN_SIGNATURE_FIELD_NUMBER: _ClassVar[int]
    account: _containers.RepeatedScalarFieldContainer[str]
    owner: _containers.RepeatedScalarFieldContainer[str]
    filters: _containers.RepeatedCompositeFieldContainer[SubscribeRequestFilterAccountsFilter]
    nonempty_txn_signature: bool
    def __init__(self, account: _Optional[_Iterable[str]] = ..., owner: _Optional[_Iterable[str]] = ..., filters: _Optional[_Iterable[_Union[SubscribeRequestFilterAccountsFilter, _Mapping]]] = ..., nonempty_txn_signature: bool = ...) -> None: ...

class SubscribeRequestFilterAccountsFilter(_message.Message):
    __slots__ = ("memcmp", "datasize", "token_account_state", "lamports")
    MEMCMP_FIELD_NUMBER: _ClassVar[int]
    DATASIZE_FIELD_NUMBER: _ClassVar[int]
    TOKEN_ACCOUNT_STATE_FIELD_NUMBER: _ClassVar[int]
    LAMPORTS_FIELD_NUMBER: _ClassVar[int]
    memcmp: SubscribeRequestFilterAccountsFilterMemcmp
    datasize: int
    token_account_state: bool
    lamports: SubscribeRequestFilterAccountsFilterLamports
    def __init__(self, memcmp: _Optional[_Union[SubscribeRequestFilterAccountsFilterMemcmp, _Mapping]] = ..., datasize: _Optional[int] = ..., token_account_state: bool = ..., lamports: _Optional[_Union[SubscribeRequestFilterAccountsFilterLamports, _Mapping]] = ...) -> None: ...

class SubscribeRequestFilterAccountsFilterMemcmp(_message.Message):
    __slots__ = ("offset", "bytes", "base58", "base64")
    OFFSET_FIELD_NUMBER: _ClassVar[int]
    BYTES_FIELD_NUMBER: _ClassVar[int]
    BASE58_FIELD_NUMBER: _ClassVar[int]
    BASE64_FIELD_NUMBER: _ClassVar[int]
    offset: int
    bytes: bytes
    base58: str
    base64: str
    def __init__(self, offset: _Optional[int] = ..., bytes: _Optional[bytes] = ..., base58: _Optional[str] = ..., base64: _Optional[str] = ...) -> None: ...

class SubscribeRequestFilterAccountsFilterLamports(_message.Message):
    __slots__ = ("eq", "ne", "lt", "gt")
    EQ_FIELD_NUMBER: _ClassVar[int]
    NE_FIELD_NUMBER: _ClassVar[int]
    LT_FIELD_NUMBER: _ClassVar[int]
    GT_FIELD_NUMBER: _ClassVar[int]
    eq: int
    ne: int
    lt: int
    gt: int
    def __init__(self, eq: _Optional[int] = ..., ne: _Optional[int] = ..., lt: _Optional[int] = ..., gt: _Optional[int] = ...) -> None: ...

class SubscribeRequestFilterSlots(_message.Message):
    __slots__ = ("filter_by_commitment", "interslot_updates")
    FILTER_BY_COMMITMENT_FIELD_NUMBER: _ClassVar[int]
    INTERSLOT_UPDATES_FIELD_NUMBER: _ClassVar[int]
    filter_by_commitment: bool
    interslot_updates: bool
    def __init__(self, filter_by_commitment: bool = ..., interslot_updates: bool = ...) -> None: ...

class SubscribeRequestFilterTransactions(_message.Message):
    __slots__ = ("vote", "failed", "signature", "account_include", "account_exclude", "account_required")
    VOTE_FIELD_NUMBER: _ClassVar[int]
    FAILED_FIELD_NUMBER: _ClassVar[int]
    SIGNATURE_FIELD_NUMBER: _ClassVar[int]
    ACCOUNT_INCLUDE_FIELD_NUMBER: _ClassVar[int]
    ACCOUNT_EXCLUDE_FIELD_NUMBER: _ClassVar[int]
    ACCOUNT_REQUIRED_FIELD_NUMBER: _ClassVar[int]
    vote: bool
    failed: bool
    signature: str
    account_include: _containers.RepeatedScalarFieldContainer[str]
    account_exclude: _containers.RepeatedScalarFieldContainer[str]
    account_required: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, vote: bool = ..., failed: bool = ..., signature: _Optional[str] = ..., account_include: _Optional[_Iterable[str]] = ..., account_exclude: _Optional[_Iterable[str]] = ..., account_required: _Optional[_Iterable[str]] = ...) -> None: ...

class SubscribeRequestFilterBlocks(_message.Message):
    __slots__ = ("account_include", "include_transactions", "include_accounts", "include_entries")
    ACCOUNT_INCLUDE_FIELD_NUMBER: _ClassVar[int]
    INCLUDE_TRANSACTIONS_FIELD_NUMBER: _ClassVar[int]
    INCLUDE_ACCOUNTS_FIELD_NUMBER: _ClassVar[int]
    INCLUDE_ENTRIES_FIELD_NUMBER: _ClassVar[int]
    account_include: _containers.RepeatedScalarFieldContainer[str]
    include_transactions: bool
    include_accounts: bool
    include_entries: bool
    def __init__(self, account_include: _Optional[_Iterable[str]] = ..., include_transactions: bool = ..., include_accounts: bool = ..., include_entries: bool = ...) -> None: ...

class SubscribeRequestFilterBlocksMeta(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class SubscribeRequestFilterEntry(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class SubscribeRequestAccountsDataSlice(_message.Message):
    __slots__ = ("offset", "length")
    OFFSET_FIELD_NUMBER: _ClassVar[int]
    LENGTH_FIELD_NUMBER: _ClassVar[int]
    offset: int
    length: int
    def __init__(self, offset: _Optional[int] = ..., length: _Optional[int] = ...) -> None: ...

class SubscribeRequestPing(_message.Message):
    __slots__ = ("id",)
    ID_FIELD_NUMBER: _ClassVar[int]
    id: int
    def __init__(self, id: _Optional[int] = ...) -> None: ...

class SubscribeUpdate(_message.Message):
    __slots__ = ("filters", "account", "slot", "transaction", "transaction_status", "block", "ping", "pong", "block_meta", "entry", "created_at")
    FILTERS_FIELD_NUMBER: _ClassVar[int]
    ACCOUNT_FIELD_NUMBER: _ClassVar[int]
    SLOT_FIELD_NUMBER: _ClassVar[int]
    TRANSACTION_FIELD_NUMBER: _ClassVar[int]
    TRANSACTION_STATUS_FIELD_NUMBER: _ClassVar[int]
    BLOCK_FIELD_NUMBER: _ClassVar[int]
    PING_FIELD_NUMBER: _ClassVar[int]
    PONG_FIELD_NUMBER: _ClassVar[int]
    BLOCK_META_FIELD_NUMBER: _ClassVar[int]
    ENTRY_FIELD_NUMBER: _ClassVar[int]
    CREATED_AT_FIELD_NUMBER: _ClassVar[int]
    filters: _containers.RepeatedScalarFieldContainer[str]
    account: SubscribeUpdateAccount
    slot: SubscribeUpdateSlot
    transaction: SubscribeUpdateTransaction
    transaction_status: SubscribeUpdateTransactionStatus
    block: SubscribeUpdateBlock
    ping: SubscribeUpdatePing
    pong: SubscribeUpdatePong
    block_meta: SubscribeUpdateBlockMeta
    entry: SubscribeUpdateEntry
    created_at: _timestamp_pb2.Timestamp
    def __init__(self, filters: _Optional[_Iterable[str]] = ..., account: _Optional[_Union[SubscribeUpdateAccount, _Mapping]] = ..., slot: _Optional[_Union[SubscribeUpdateSlot, _Mapping]] = ..., transaction: _Optional[_Union[SubscribeUpdateTransaction, _Mapping]] = ..., transaction_status: _Optional[_Union[SubscribeUpdateTransactionStatus, _Mapping]] = ..., block: _Optional[_Union[SubscribeUpdateBlock, _Mapping]] = ..., ping: _Optional[_Union[SubscribeUpdatePing, _Mapping]] = ..., pong: _Optional[_Union[SubscribeUpdatePong, _Mapping]] = ..., block_meta: _Optional[_Union[SubscribeUpdateBlockMeta, _Mapping]] = ..., entry: _Optional[_Union[SubscribeUpdateEntry, _Mapping]] = ..., created_at: _Optional[_Union[datetime.datetime, _timestamp_pb2.Timestamp, _Mapping]] = ...) -> None: ...

class SubscribeUpdateAccount(_message.Message):
    __slots__ = ("account", "slot", "is_startup")
    ACCOUNT_FIELD_NUMBER: _ClassVar[int]
    SLOT_FIELD_NUMBER: _ClassVar[int]
    IS_STARTUP_FIELD_NUMBER: _ClassVar[int]
    account: SubscribeUpdateAccountInfo
    slot: int
    is_startup: bool
    def __init__(self, account: _Optional[_Union[SubscribeUpdateAccountInfo, _Mapping]] = ..., slot: _Optional[int] = ..., is_startup: bool = ...) -> None: ...

class SubscribeUpdateAccountInfo(_message.Message):
    __slots__ = ("pubkey", "lamports", "owner", "executable", "rent_epoch", "data", "write_version", "txn_signature")
    PUBKEY_FIELD_NUMBER: _ClassVar[int]
    LAMPORTS_FIELD_NUMBER: _ClassVar[int]
    OWNER_FIELD_NUMBER: _ClassVar[int]
    EXECUTABLE_FIELD_NUMBER: _ClassVar[int]
    RENT_EPOCH_FIELD_NUMBER: _ClassVar[int]
    DATA_FIELD_NUMBER: _ClassVar[int]
    WRITE_VERSION_FIELD_NUMBER: _ClassVar[int]
    TXN_SIGNATURE_FIELD_NUMBER: _ClassVar[int]
    pubkey: bytes
    lamports: int
    owner: bytes
    executable: bool
    rent_epoch: int
    data: bytes
    write_version: int
    txn_signature: bytes
    def __init__(self, pubkey: _Optional[bytes] = ..., lamports: _Optional[int] = ..., owner: _Optional[bytes] = ..., executable: bool = ..., rent_epoch: _Optional[int] = ..., data: _Optional[bytes] = ..., write_version: _Optional[int] = ..., txn_signature: _Optional[bytes] = ...) -> None: ...

class SubscribeUpdateSlot(_message.Message):
    __slots__ = ("slot", "parent", "status", "dead_error")
    SLOT_FIELD_NUMBER: _ClassVar[int]
    PARENT_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    DEAD_ERROR_FIELD_NUMBER: _ClassVar[int]
    slot: int
    parent: int
    status: SlotStatus
    dead_error: str
    def __init__(self, slot: _Optional[int] = ..., parent: _Optional[int] = ..., status: _Optional[_Union[SlotStatus, str]] = ..., dead_error: _Optional[str] = ...) -> None: ...

class SubscribeUpdateTransaction(_message.Message):
    __slots__ = ("transaction", "slot")
    TRANSACTION_FIELD_NUMBER: _ClassVar[int]
    SLOT_FIELD_NUMBER: _ClassVar[int]
    transaction: SubscribeUpdateTransactionInfo
    slot: int
    def __init__(self, transaction: _Optional[_Union[SubscribeUpdateTransactionInfo, _Mapping]] = ..., slot: _Optional[int] = ...) -> None: ...

class SubscribeUpdateTransactionInfo(_message.Message):
    __slots__ = ("signature", "is_vote", "transaction", "meta", "index")
    SIGNATURE_FIELD_NUMBER: _ClassVar[int]
    IS_VOTE_FIELD_NUMBER: _ClassVar[int]
    TRANSACTION_FIELD_NUMBER: _ClassVar[int]
    META_FIELD_NUMBER: _ClassVar[int]
    INDEX_FIELD_NUMBER: _ClassVar[int]
    signature: bytes
    is_vote: bool
    transaction: _solana_storage_pb2.Transaction
    meta: _solana_storage_pb2.TransactionStatusMeta
    index: int
    def __init__(self, signature: _Optional[bytes] = ..., is_vote: bool = ..., transaction: _Optional[_Union[_solana_storage_pb2.Transaction, _Mapping]] = ..., meta: _Optional[_Union[_solana_storage_pb2.TransactionStatusMeta, _Mapping]] = ..., index: _Optional[int] = ...) -> None: ...

class SubscribeUpdateTransactionStatus(_message.Message):
    __slots__ = ("slot", "signature", "is_vote", "index", "err")
    SLOT_FIELD_NUMBER: _ClassVar[int]
    SIGNATURE_FIELD_NUMBER: _ClassVar[int]
    IS_VOTE_FIELD_NUMBER: _ClassVar[int]
    INDEX_FIELD_NUMBER: _ClassVar[int]
    ERR_FIELD_NUMBER: _ClassVar[int]
    slot: int
    signature: bytes
    is_vote: bool
    index: int
    err: _solana_storage_pb2.TransactionError
    def __init__(self, slot: _Optional[int] = ..., signature: _Optional[bytes] = ..., is_vote: bool = ..., index: _Optional[int] = ..., err: _Optional[_Union[_solana_storage_pb2.TransactionError, _Mapping]] = ...) -> None: ...

class SubscribeUpdateBlock(_message.Message):
    __slots__ = ("slot", "blockhash", "rewards", "block_time", "block_height", "parent_slot", "parent_blockhash", "executed_transaction_count", "transactions", "updated_account_count", "accounts", "entries_count", "entries")
    SLOT_FIELD_NUMBER: _ClassVar[int]
    BLOCKHASH_FIELD_NUMBER: _ClassVar[int]
    REWARDS_FIELD_NUMBER: _ClassVar[int]
    BLOCK_TIME_FIELD_NUMBER: _ClassVar[int]
    BLOCK_HEIGHT_FIELD_NUMBER: _ClassVar[int]
    PARENT_SLOT_FIELD_NUMBER: _ClassVar[int]
    PARENT_BLOCKHASH_FIELD_NUMBER: _ClassVar[int]
    EXECUTED_TRANSACTION_COUNT_FIELD_NUMBER: _ClassVar[int]
    TRANSACTIONS_FIELD_NUMBER: _ClassVar[int]
    UPDATED_ACCOUNT_COUNT_FIELD_NUMBER: _ClassVar[int]
    ACCOUNTS_FIELD_NUMBER: _ClassVar[int]
    ENTRIES_COUNT_FIELD_NUMBER: _ClassVar[int]
    ENTRIES_FIELD_NUMBER: _ClassVar[int]
    slot: int
    blockhash: str
    rewards: _solana_storage_pb2.Rewards
    block_time: _solana_storage_pb2.UnixTimestamp
    block_height: _solana_storage_pb2.BlockHeight
    parent_slot: int
    parent_blockhash: str
    executed_transaction_count: int
    transactions: _containers.RepeatedCompositeFieldContainer[SubscribeUpdateTransactionInfo]
    updated_account_count: int
    accounts: _containers.RepeatedCompositeFieldContainer[SubscribeUpdateAccountInfo]
    entries_count: int
    entries: _containers.RepeatedCompositeFieldContainer[SubscribeUpdateEntry]
    def __init__(self, slot: _Optional[int] = ..., blockhash: _Optional[str] = ..., rewards: _Optional[_Union[_solana_storage_pb2.Rewards, _Mapping]] = ..., block_time: _Optional[_Union[_solana_storage_pb2.UnixTimestamp, _Mapping]] = ..., block_height: _Optional[_Union[_solana_storage_pb2.BlockHeight, _Mapping]] = ..., parent_slot: _Optional[int] = ..., parent_blockhash: _Optional[str] = ..., executed_transaction_count: _Optional[int] = ..., transactions: _Optional[_Iterable[_Union[SubscribeUpdateTransactionInfo, _Mapping]]] = ..., updated_account_count: _Optional[int] = ..., accounts: _Optional[_Iterable[_Union[SubscribeUpdateAccountInfo, _Mapping]]] = ..., entries_count: _Optional[int] = ..., entries: _Optional[_Iterable[_Union[SubscribeUpdateEntry, _Mapping]]] = ...) -> None: ...

class SubscribeUpdateBlockMeta(_message.Message):
    __slots__ = ("slot", "blockhash", "rewards", "block_time", "block_height", "parent_slot", "parent_blockhash", "executed_transaction_count", "entries_count")
    SLOT_FIELD_NUMBER: _ClassVar[int]
    BLOCKHASH_FIELD_NUMBER: _ClassVar[int]
    REWARDS_FIELD_NUMBER: _ClassVar[int]
    BLOCK_TIME_FIELD_NUMBER: _ClassVar[int]
    BLOCK_HEIGHT_FIELD_NUMBER: _ClassVar[int]
    PARENT_SLOT_FIELD_NUMBER: _ClassVar[int]
    PARENT_BLOCKHASH_FIELD_NUMBER: _ClassVar[int]
    EXECUTED_TRANSACTION_COUNT_FIELD_NUMBER: _ClassVar[int]
    ENTRIES_COUNT_FIELD_NUMBER: _ClassVar[int]
    slot: int
    blockhash: str
    rewards: _solana_storage_pb2.Rewards
    block_time: _solana_storage_pb2.UnixTimestamp
    block_height: _solana_storage_pb2.BlockHeight
    parent_slot: int
    parent_blockhash: str
    executed_transaction_count: int
    entries_count: int
    def __init__(self, slot: _Optional[int] = ..., blockhash: _Optional[str] = ..., rewards: _Optional[_Union[_solana_storage_pb2.Rewards, _Mapping]] = ..., block_time: _Optional[_Union[_solana_storage_pb2.UnixTimestamp, _Mapping]] = ..., block_height: _Optional[_Union[_solana_storage_pb2.BlockHeight, _Mapping]] = ..., parent_slot: _Optional[int] = ..., parent_blockhash: _Optional[str] = ..., executed_transaction_count: _Optional[int] = ..., entries_count: _Optional[int] = ...) -> None: ...

class SubscribeUpdateEntry(_message.Message):
    __slots__ = ("slot", "index", "num_hashes", "hash", "executed_transaction_count", "starting_transaction_index")
    SLOT_FIELD_NUMBER: _ClassVar[int]
    INDEX_FIELD_NUMBER: _ClassVar[int]
    NUM_HASHES_FIELD_NUMBER: _ClassVar[int]
    HASH_FIELD_NUMBER: _ClassVar[int]
    EXECUTED_TRANSACTION_COUNT_FIELD_NUMBER: _ClassVar[int]
    STARTING_TRANSACTION_INDEX_FIELD_NUMBER: _ClassVar[int]
    slot: int
    index: int
    num_hashes: int
    hash: bytes
    executed_transaction_count: int
    starting_transaction_index: int
    def __init__(self, slot: _Optional[int] = ..., index: _Optional[int] = ..., num_hashes: _Optional[int] = ..., hash: _Optional[bytes] = ..., executed_transaction_count: _Optional[int] = ..., starting_transaction_index: _Optional[int] = ...) -> None: ...

class SubscribeUpdatePing(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class SubscribeUpdatePong(_message.Message):
    __slots__ = ("id",)
    ID_FIELD_NUMBER: _ClassVar[int]
    id: int
    def __init__(self, id: _Optional[int] = ...) -> None: ...

class SubscribeReplayInfoRequest(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class SubscribeReplayInfoResponse(_message.Message):
    __slots__ = ("first_available",)
    FIRST_AVAILABLE_FIELD_NUMBER: _ClassVar[int]
    first_available: int
    def __init__(self, first_available: _Optional[int] = ...) -> None: ...

class PingRequest(_message.Message):
    __slots__ = ("count",)
    COUNT_FIELD_NUMBER: _ClassVar[int]
    count: int
    def __init__(self, count: _Optional[int] = ...) -> None: ...

class PongResponse(_message.Message):
    __slots__ = ("count",)
    COUNT_FIELD_NUMBER: _ClassVar[int]
    count: int
    def __init__(self, count: _Optional[int] = ...) -> None: ...

class GetLatestBlockhashRequest(_message.Message):
    __slots__ = ("commitment",)
    COMMITMENT_FIELD_NUMBER: _ClassVar[int]
    commitment: CommitmentLevel
    def __init__(self, commitment: _Optional[_Union[CommitmentLevel, str]] = ...) -> None: ...

class GetLatestBlockhashResponse(_message.Message):
    __slots__ = ("slot", "blockhash", "last_valid_block_height")
    SLOT_FIELD_NUMBER: _ClassVar[int]
    BLOCKHASH_FIELD_NUMBER: _ClassVar[int]
    LAST_VALID_BLOCK_HEIGHT_FIELD_NUMBER: _ClassVar[int]
    slot: int
    blockhash: str
    last_valid_block_height: int
    def __init__(self, slot: _Optional[int] = ..., blockhash: _Optional[str] = ..., last_valid_block_height: _Optional[int] = ...) -> None: ...

class GetBlockHeightRequest(_message.Message):
    __slots__ = ("commitment",)
    COMMITMENT_FIELD_NUMBER: _ClassVar[int]
    commitment: CommitmentLevel
    def __init__(self, commitment: _Optional[_Union[CommitmentLevel, str]] = ...) -> None: ...

class GetBlockHeightResponse(_message.Message):
    __slots__ = ("block_height",)
    BLOCK_HEIGHT_FIELD_NUMBER: _ClassVar[int]
    block_height: int
    def __init__(self, block_height: _Optional[int] = ...) -> None: ...

class GetSlotRequest(_message.Message):
    __slots__ = ("commitment",)
    COMMITMENT_FIELD_NUMBER: _ClassVar[int]
    commitment: CommitmentLevel
    def __init__(self, commitment: _Optional[_Union[CommitmentLevel, str]] = ...) -> None: ...

class GetSlotResponse(_message.Message):
    __slots__ = ("slot",)
    SLOT_FIELD_NUMBER: _ClassVar[int]
    slot: int
    def __init__(self, slot: _Optional[int] = ...) -> None: ...

class GetVersionRequest(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class GetVersionResponse(_message.Message):
    __slots__ = ("version",)
    VERSION_FIELD_NUMBER: _ClassVar[int]
    version: str
    def __init__(self, version: _Optional[str] = ...) -> None: ...

class IsBlockhashValidRequest(_message.Message):
    __slots__ = ("blockhash", "commitment")
    BLOCKHASH_FIELD_NUMBER: _ClassVar[int]
    COMMITMENT_FIELD_NUMBER: _ClassVar[int]
    blockhash: str
    commitment: CommitmentLevel
    def __init__(self, blockhash: _Optional[str] = ..., commitment: _Optional[_Union[CommitmentLevel, str]] = ...) -> None: ...

class IsBlockhashValidResponse(_message.Message):
    __slots__ = ("slot", "valid")
    SLOT_FIELD_NUMBER: _ClassVar[int]
    VALID_FIELD_NUMBER: _ClassVar[int]
    slot: int
    valid: bool
    def __init__(self, slot: _Optional[int] = ..., valid: bool = ...) -> None: ...



from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from collections.abc import Iterable as _Iterable, Mapping as _Mapping
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class RewardType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    Unspecified: _ClassVar[RewardType]
    Fee: _ClassVar[RewardType]
    Rent: _ClassVar[RewardType]
    Staking: _ClassVar[RewardType]
    Voting: _ClassVar[RewardType]
Unspecified: RewardType
Fee: RewardType
Rent: RewardType
Staking: RewardType
Voting: RewardType

class ConfirmedBlock(_message.Message):
    __slots__ = ("previous_blockhash", "blockhash", "parent_slot", "transactions", "rewards", "block_time", "block_height", "num_partitions")
    PREVIOUS_BLOCKHASH_FIELD_NUMBER: _ClassVar[int]
    BLOCKHASH_FIELD_NUMBER: _ClassVar[int]
    PARENT_SLOT_FIELD_NUMBER: _ClassVar[int]
    TRANSACTIONS_FIELD_NUMBER: _ClassVar[int]
    REWARDS_FIELD_NUMBER: _ClassVar[int]
    BLOCK_TIME_FIELD_NUMBER: _ClassVar[int]
    BLOCK_HEIGHT_FIELD_NUMBER: _ClassVar[int]
    NUM_PARTITIONS_FIELD_NUMBER: _ClassVar[int]
    previous_blockhash: str
    blockhash: str
    parent_slot: int
    transactions: _containers.RepeatedCompositeFieldContainer[ConfirmedTransaction]
    rewards: _containers.RepeatedCompositeFieldContainer[Reward]
    block_time: UnixTimestamp
    block_height: BlockHeight
    num_partitions: NumPartitions
    def __init__(self, previous_blockhash: _Optional[str] = ..., blockhash: _Optional[str] = ..., parent_slot: _Optional[int] = ..., transactions: _Optional[_Iterable[_Union[ConfirmedTransaction, _Mapping]]] = ..., rewards: _Optional[_Iterable[_Union[Reward, _Mapping]]] = ..., block_time: _Optional[_Union[UnixTimestamp, _Mapping]] = ..., block_height: _Optional[_Union[BlockHeight, _Mapping]] = ..., num_partitions: _Optional[_Union[NumPartitions, _Mapping]] = ...) -> None: ...

class ConfirmedTransaction(_message.Message):
    __slots__ = ("transaction", "meta")
    TRANSACTION_FIELD_NUMBER: _ClassVar[int]
    META_FIELD_NUMBER: _ClassVar[int]
    transaction: Transaction
    meta: TransactionStatusMeta
    def __init__(self, transaction: _Optional[_Union[Transaction, _Mapping]] = ..., meta: _Optional[_Union[TransactionStatusMeta, _Mapping]] = ...) -> None: ...

class Transaction(_message.Message):
    __slots__ = ("signatures", "message")
    SIGNATURES_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    signatures: _containers.RepeatedScalarFieldContainer[bytes]
    message: Message
    def __init__(self, signatures: _Optional[_Iterable[bytes]] = ..., message: _Optional[_Union[Message, _Mapping]] = ...) -> None: ...

class Message(_message.Message):
    __slots__ = ("header", "account_keys", "recent_blockhash", "instructions", "versioned", "address_table_lookups")
    HEADER_FIELD_NUMBER: _ClassVar[int]
    ACCOUNT_KEYS_FIELD_NUMBER: _ClassVar[int]
    RECENT_BLOCKHASH_FIELD_NUMBER: _ClassVar[int]
    INSTRUCTIONS_FIELD_NUMBER: _ClassVar[int]
    VERSIONED_FIELD_NUMBER: _ClassVar[int]
    ADDRESS_TABLE_LOOKUPS_FIELD_NUMBER: _ClassVar[int]
    header: MessageHeader
    account_keys: _containers.RepeatedScalarFieldContainer[bytes]
    recent_blockhash: bytes
    instructions: _containers.RepeatedCompositeFieldContainer[CompiledInstruction]
    versioned: bool
    address_table_lookups: _containers.RepeatedCompositeFieldContainer[MessageAddressTableLookup]
    def __init__(self, header: _Optional[_Union[MessageHeader, _Mapping]] = ..., account_keys: _Optional[_Iterable[bytes]] = ..., recent_blockhash: _Optional[bytes] = ..., instructions: _Optional[_Iterable[_Union[CompiledInstruction, _Mapping]]] = ..., versioned: bool = ..., address_table_lookups: _Optional[_Iterable[_Union[MessageAddressTableLookup, _Mapping]]] = ...) -> None: ...

class MessageHeader(_message.Message):
    __slots__ = ("num_required_signatures", "num_readonly_signed_accounts", "num_readonly_unsigned_accounts")
    NUM_REQUIRED_SIGNATURES_FIELD_NUMBER: _ClassVar[int]
    NUM_READONLY_SIGNED_ACCOUNTS_FIELD_NUMBER: _ClassVar[int]
    NUM_READONLY_UNSIGNED_ACCOUNTS_FIELD_NUMBER: _ClassVar[int]
    num_required_signatures: int
    num_readonly_signed_accounts: int
    num_readonly_unsigned_accounts: int
    def __init__(self, num_required_signatures: _Optional[int] = ..., num_readonly_signed_accounts: _Optional[int] = ..., num_readonly_unsigned_accounts: _Optional[int] = ...) -> None: ...

class MessageAddressTableLookup(_message.Message):
    __slots__ = ("account_key", "writable_indexes", "readonly_indexes")
    ACCOUNT_KEY_FIELD_NUMBER: _ClassVar[int]
    WRITABLE_INDEXES_FIELD_NUMBER: _ClassVar[int]
    READONLY_INDEXES_FIELD_NUMBER: _ClassVar[int]
    account_key: bytes
    writable_indexes: bytes
    readonly_indexes: bytes
    def __init__(self, account_key: _Optional[bytes] = ..., writable_indexes: _Optional[bytes] = ..., readonly_indexes: _Optional[bytes] = ...) -> None: ...

class TransactionStatusMeta(_message.Message):
    __slots__ = ("err", "fee", "pre_balances", "post_balances", "inner_instructions", "inner_instructions_none", "log_messages", "log_messages_none", "pre_token_balances", "post_token_balances", "rewards", "loaded_writable_addresses", "loaded_readonly_addresses", "return_data", "return_data_none", "compute_units_consumed", "cost_units")
    ERR_FIELD_NUMBER: _ClassVar[int]
    FEE_FIELD_NUMBER: _ClassVar[int]
    PRE_BALANCES_FIELD_NUMBER: _ClassVar[int]
    POST_BALANCES_FIELD_NUMBER: _ClassVar[int]
    INNER_INSTRUCTIONS_FIELD_NUMBER: _ClassVar[int]
    INNER_INSTRUCTIONS_NONE_FIELD_NUMBER: _ClassVar[int]
    LOG_MESSAGES_FIELD_NUMBER: _ClassVar[int]
    LOG_MESSAGES_NONE_FIELD_NUMBER: _ClassVar[int]
    PRE_TOKEN_BALANCES_FIELD_NUMBER: _ClassVar[int]
    POST_TOKEN_BALANCES_FIELD_NUMBER: _ClassVar[int]
    REWARDS_FIELD_NUMBER: _ClassVar[int]
    LOADED_WRITABLE_ADDRESSES_FIELD_NUMBER: _ClassVar[int]
    LOADED_READONLY_ADDRESSES_FIELD_NUMBER: _ClassVar[int]
    RETURN_DATA_FIELD_NUMBER: _ClassVar[int]
    RETURN_DATA_NONE_FIELD_NUMBER: _ClassVar[int]
    COMPUTE_UNITS_CONSUMED_FIELD_NUMBER: _ClassVar[int]
    COST_UNITS_FIELD_NUMBER: _ClassVar[int]
    err: TransactionError
    fee: int
    pre_balances: _containers.RepeatedScalarFieldContainer[int]
    post_balances: _containers.RepeatedScalarFieldContainer[int]
    inner_instructions: _containers.RepeatedCompositeFieldContainer[InnerInstructions]
    inner_instructions_none: bool
    log_messages: _containers.RepeatedScalarFieldContainer[str]
    log_messages_none: bool
    pre_token_balances: _containers.RepeatedCompositeFieldContainer[TokenBalance]
    post_token_balances: _containers.RepeatedCompositeFieldContainer[TokenBalance]
    rewards: _containers.RepeatedCompositeFieldContainer[Reward]
    loaded_writable_addresses: _containers.RepeatedScalarFieldContainer[bytes]
    loaded_readonly_addresses: _containers.RepeatedScalarFieldContainer[bytes]
    return_data: ReturnData
    return_data_none: bool
    compute_units_consumed: int
    cost_units: int
    def __init__(self, err: _Optional[_Union[TransactionError, _Mapping]] = ..., fee: _Optional[int] = ..., pre_balances: _Optional[_Iterable[int]] = ..., post_balances: _Optional[_Iterable[int]] = ..., inner_instructions: _Optional[_Iterable[_Union[InnerInstructions, _Mapping]]] = ..., inner_instructions_none: bool = ..., log_messages: _Optional[_Iterable[str]] = ..., log_messages_none: bool = ..., pre_token_balances: _Optional[_Iterable[_Union[TokenBalance, _Mapping]]] = ..., post_token_balances: _Optional[_Iterable[_Union[TokenBalance, _Mapping]]] = ..., rewards: _Optional[_Iterable[_Union[Reward, _Mapping]]] = ..., loaded_writable_addresses: _Optional[_Iterable[bytes]] = ..., loaded_readonly_addresses: _Optional[_Iterable[bytes]] = ..., return_data: _Optional[_Union[ReturnData, _Mapping]] = ..., return_data_none: bool = ..., compute_units_consumed: _Optional[int] = ..., cost_units: _Optional[int] = ...) -> None: ...

class TransactionError(_message.Message):
    __slots__ = ("err",)
    ERR_FIELD_NUMBER: _ClassVar[int]
    err: bytes
    def __init__(self, err: _Optional[bytes] = ...) -> None: ...

class InnerInstructions(_message.Message):
    __slots__ = ("index", "instructions")
    INDEX_FIELD_NUMBER: _ClassVar[int]
    INSTRUCTIONS_FIELD_NUMBER: _ClassVar[int]
    index: int
    instructions: _containers.RepeatedCompositeFieldContainer[InnerInstruction]
    def __init__(self, index: _Optional[int] = ..., instructions: _Optional[_Iterable[_Union[InnerInstruction, _Mapping]]] = ...) -> None: ...

class InnerInstruction(_message.Message):
    __slots__ = ("program_id_index", "accounts", "data", "stack_height")
    PROGRAM_ID_INDEX_FIELD_NUMBER: _ClassVar[int]
    ACCOUNTS_FIELD_NUMBER: _ClassVar[int]
    DATA_FIELD_NUMBER: _ClassVar[int]
    STACK_HEIGHT_FIELD_NUMBER: _ClassVar[int]
    program_id_index: int
    accounts: bytes
    data: bytes
    stack_height: int
    def __init__(self, program_id_index: _Optional[int] = ..., accounts: _Optional[bytes] = ..., data: _Optional[bytes] = ..., stack_height: _Optional[int] = ...) -> None: ...

class CompiledInstruction(_message.Message):
    __slots__ = ("program_id_index", "accounts", "data")
    PROGRAM_ID_INDEX_FIELD_NUMBER: _ClassVar[int]
    ACCOUNTS_FIELD_NUMBER: _ClassVar[int]
    DATA_FIELD_NUMBER: _ClassVar[int]
    program_id_index: int
    accounts: bytes
    data: bytes
    def __init__(self, program_id_index: _Optional[int] = ..., accounts: _Optional[bytes] = ..., data: _Optional[bytes] = ...) -> None: ...

class TokenBalance(_message.Message):
    __slots__ = ("account_index", "mint", "ui_token_amount", "owner", "program_id")
    ACCOUNT_INDEX_FIELD_NUMBER: _ClassVar[int]
    MINT_FIELD_NUMBER: _ClassVar[int]
    UI_TOKEN_AMOUNT_FIELD_NUMBER: _ClassVar[int]
    OWNER_FIELD_NUMBER: _ClassVar[int]
    PROGRAM_ID_FIELD_NUMBER: _ClassVar[int]
    account_index: int
    mint: str
    ui_token_amount: UiTokenAmount
    owner: str
    program_id: str
    def __init__(self, account_index: _Optional[int] = ..., mint: _Optional[str] = ..., ui_token_amount: _Optional[_Union[UiTokenAmount, _Mapping]] = ..., owner: _Optional[str] = ..., program_id: _Optional[str] = ...) -> None: ...

class UiTokenAmount(_message.Message):
    __slots__ = ("ui_amount", "decimals", "amount", "ui_amount_string")
    UI_AMOUNT_FIELD_NUMBER: _ClassVar[int]
    DECIMALS_FIELD_NUMBER: _ClassVar[int]
    AMOUNT_FIELD_NUMBER: _ClassVar[int]
    UI_AMOUNT_STRING_FIELD_NUMBER: _ClassVar[int]
    ui_amount: float
    decimals: int
    amount: str
    ui_amount_string: str
    def __init__(self, ui_amount: _Optional[float] = ..., decimals: _Optional[int] = ..., amount: _Optional[str] = ..., ui_amount_string: _Optional[str] = ...) -> None: ...

class ReturnData(_message.Message):
    __slots__ = ("program_id", "data")
    PROGRAM_ID_FIELD_NUMBER: _ClassVar[int]
    DATA_FIELD_NUMBER: _ClassVar[int]
    program_id: bytes
    data: bytes
    def __init__(self, program_id: _Optional[bytes] = ..., data: _Optional[bytes] = ...) -> None: ...

class Reward(_message.Message):
    __slots__ = ("pubkey", "lamports", "post_balance", "reward_type", "commission")
    PUBKEY_FIELD_NUMBER: _ClassVar[int]
    LAMPORTS_FIELD_NUMBER: _ClassVar[int]
    POST_BALANCE_FIELD_NUMBER: _ClassVar[int]
    REWARD_TYPE_FIELD_NUMBER: _ClassVar[int]
    COMMISSION_FIELD_NUMBER: _ClassVar[int]
    pubkey: str
    lamports: int
    post_balance: int
    reward_type: RewardType
    commission: str
    def __init__(self, pubkey: _Optional[str] = ..., lamports: _Optional[int] = ..., post_balance: _Optional[int] = ..., reward_type: _Optional[_Union[RewardType, str]] = ..., commission: _Optional[str] = ...) -> None: ...

class Rewards(_message.Message):
    __slots__ = ("rewards", "num_partitions")
    REWARDS_FIELD_NUMBER: _ClassVar[int]
    NUM_PARTITIONS_FIELD_NUMBER: _ClassVar[int]
    rewards: _containers.RepeatedCompositeFieldContainer[Reward]
    num_partitions: NumPartitions
    def __init__(self, rewards: _Optional[_Iterable[_Union[Reward, _Mapping]]] = ..., num_partitions: _Optional[_Union[NumPartitions, _Mapping]] = ...) -> None: ...

class UnixTimestamp(_message.Message):
    __slots__ = ("timestamp",)
    TIMESTAMP_FIELD_NUMBER: _ClassVar[int]
    timestamp: int
    def __init__(self, timestamp: _Optional[int] = ...) -> None: ...

class BlockHeight(_message.Message):
    __slots__ = ("block_height",)
    BLOCK_HEIGHT_FIELD_NUMBER: _ClassVar[int]
    block_height: int
    def __init__(self, block_height: _Optional[int] = ...) -> None: ...

class NumPartitions(_message.Message):
    __slots__ = ("num_partitions",)
    NUM_PARTITIONS_FIELD_NUMBER: _ClassVar[int]
    num_partitions: int
    def __init__(self, num_partitions: _Optional[int] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: fumarole.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'fumarole.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import yellowstone_fumarole_proto.geyser_pb2 as geyser__pb2
try:
  solana__storage__pb2 = geyser__pb2.solana__storage__pb2
except AttributeError:
  solana__storage__pb2 = geyser__pb2.solana_storage_pb2

from yellowstone_fumarole_proto.geyser_pb2 import *

DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0e\x66umarole.proto\x12\x08\x66umarole\x1a\x0cgeyser.proto\",\n\x13GetSlotRangeRequest\x12\x15\n\rblockchain_id\x18\x01 \x01(\x0c\"Q\n\x14GetSlotRangeResponse\x12\x15\n\rblockchain_id\x18\x01 \x01(\x0c\x12\x10\n\x08min_slot\x18\x02 \x01(\x03\x12\x10\n\x08max_slot\x18\x03 \x01(\x03\"+\n\x12GetChainTipRequest\x12\x15\n\rblockchain_id\x18\x01 \x01(\x0c\"\xc1\x01\n\x13GetChainTipResponse\x12\x15\n\rblockchain_id\x18\x01 \x01(\x0c\x12W\n\x17shard_to_max_offset_map\x18\x02 \x03(\x0b\x32\x36.fumarole.GetChainTipResponse.ShardToMaxOffsetMapEntry\x1a:\n\x18ShardToMaxOffsetMapEntry\x12\x0b\n\x03key\x18\x01 \x01(\x05\x12\r\n\x05value\x18\x02 \x01(\x03:\x02\x38\x01\"\x10\n\x0eVersionRequest\"\"\n\x0fVersionResponse\x12\x0f\n\x07version\x18\x01 \x01(\t\":\n\x1bGetConsumerGroupInfoRequest\x12\x1b\n\x13\x63onsumer_group_name\x18\x01 \x01(\t\"9\n\x1a\x44\x65leteConsumerGroupRequest\x12\x1b\n\x13\x63onsumer_group_name\x18\x01 \x01(\t\".\n\x1b\x44\x65leteConsumerGroupResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\"\x1b\n\x19ListConsumerGroupsRequest\"R\n\x1aListConsumerGroupsResponse\x12\x34\n\x0f\x63onsumer_groups\x18\x01 \x03(\x0b\x32\x1b.fumarole.ConsumerGroupInfo\"e\n\x11\x43onsumerGroupInfo\x12\n\n\x02id\x18\x01 \x01(\t\x12\x1b\n\x13\x63onsumer_group_name\x18\x02 \x01(\t\x12\x10\n\x08is_stale\x18\x03 \x01(\x08\x12\x15\n\rblockchain_id\x18\x04 \x01(\x0c\"4\n\x15GetSlotLagInfoRequest\x12\x1b\n\x13\x63onsumer_group_name\x18\x01 \x01(\t\"\xe5\x04\n\x0c\x42lockFilters\x12\x36\n\x08\x61\x63\x63ounts\x18\x01 \x03(\x0b\x32$.fumarole.BlockFilters.AccountsEntry\x12>\n\x0ctransactions\x18\x02 \x03(\x0b\x32(.fumarole.BlockFilters.TransactionsEntry\x12\x34\n\x07\x65ntries\x18\x03 \x03(\x0b\x32#.fumarole.BlockFilters.EntriesEntry\x12;\n\x0b\x62locks_meta\x18\x04 \x03(\x0b\x32&.fumarole.BlockFilters.BlocksMetaEntry\x1aW\n\rAccountsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x35\n\x05value\x18\x02 \x01(\x0b\x32&.geyser.SubscribeRequestFilterAccounts:\x02\x38\x01\x1a_\n\x11TransactionsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x39\n\x05value\x18\x02 \x01(\x0b\x32*.geyser.SubscribeRequestFilterTransactions:\x02\x38\x01\x1aS\n\x0c\x45ntriesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x32\n\x05value\x18\x02 \x01(\x0b\x32#.geyser.SubscribeRequestFilterEntry:\x02\x38\x01\x1a[\n\x0f\x42locksMetaEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x37\n\x05value\x18\x02 \x01(\x0b\x32(.geyser.SubscribeRequestFilterBlocksMeta:\x02\x38\x01\"\x95\x01\n\x12\x44ownloadBlockShard\x12\x15\n\rblockchain_id\x18\x01 \x01(\x0c\x12\x11\n\tblock_uid\x18\x02 \x01(\x0c\x12\x11\n\tshard_idx\x18\x03 \x01(\x05\x12\x31\n\x0c\x62lockFilters\x18\x04 \x01(\x0b\x32\x16.fumarole.BlockFiltersH\x00\x88\x01\x01\x42\x0f\n\r_blockFilters\"\x17\n\x04Ping\x12\x0f\n\x07ping_id\x18\x01 \x01(\r\"\x17\n\x04Pong\x12\x0f\n\x07ping_id\x18\x01 \x01(\r\"\x87\x01\n\x0b\x44\x61taCommand\x12<\n\x14\x64ownload_block_shard\x18\x01 \x01(\x0b\x32\x1c.fumarole.DownloadBlockShardH\x00\x12/\n\rfilter_update\x18\x02 \x01(\x0b\x32\x16.fumarole.BlockFiltersH\x00\x42\t\n\x07\x63ommand\"\x1a\n\x18\x42lockShardDownloadFinish\"L\n\rBlockNotFound\x12\x15\n\rblockchain_id\x18\x01 \x01(\x0c\x12\x11\n\tblock_uid\x18\x02 \x01(\x0c\x12\x11\n\tshard_idx\x18\x03 \x01(\x05\"B\n\tDataError\x12,\n\tnot_found\x18\x01 \x01(\x0b\x32\x17.fumarole.BlockNotFoundH\x00\x42\x07\n\x05\x65rror\"\x90\x01\n\x0c\x44\x61taResponse\x12)\n\x06update\x18\x01 \x01(\x0b\x32\x17.geyser.SubscribeUpdateH\x00\x12I\n\x1b\x62lock_shard_download_finish\x18\x02 \x01(\x0b\x32\".fumarole.BlockShardDownloadFinishH\x00\x42\n\n\x08response\"0\n\x0c\x43ommitOffset\x12\x0e\n\x06offset\x18\x01 \x01(\x03\x12\x10\n\x08shard_id\x18\x02 \x01(\x05\"c\n\x15PollBlockchainHistory\x12\x10\n\x08shard_id\x18\x01 \x01(\x05\x12\x11\n\x04\x66rom\x18\x02 \x01(\x03H\x00\x88\x01\x01\x12\x12\n\x05limit\x18\x03 \x01(\x03H\x01\x88\x01\x01\x42\x07\n\x05_fromB\x08\n\x06_limit\"\x8f\x02\n\x0f\x42lockchainEvent\x12\x0e\n\x06offset\x18\x01 \x01(\x03\x12\x15\n\rblockchain_id\x18\x02 \x01(\x0c\x12\x11\n\tblock_uid\x18\x03 \x01(\x0c\x12\x12\n\nnum_shards\x18\x04 \x01(\r\x12\x0c\n\x04slot\x18\x05 \x01(\x04\x12\x18\n\x0bparent_slot\x18\x06 \x01(\x04H\x00\x88\x01\x01\x12\x31\n\x10\x63ommitment_level\x18\x07 \x01(\x0e\x32\x17.geyser.CommitmentLevel\x12\x1b\n\x13\x62lockchain_shard_id\x18\x08 \x01(\x05\x12\x17\n\ndead_error\x18\t \x01(\tH\x01\x88\x01\x01\x42\x0e\n\x0c_parent_slotB\r\n\x0b_dead_error\">\n\x11\x42lockchainHistory\x12)\n\x06\x65vents\x18\x01 \x03(\x0b\x32\x19.fumarole.BlockchainEvent\"L\n\x10JoinControlPlane\x12 \n\x13\x63onsumer_group_name\x18\x01 \x01(\tH\x00\x88\x01\x01\x42\x16\n\x14_consumer_group_name\"\xd6\x01\n\x0e\x43ontrolCommand\x12\x32\n\x0cinitial_join\x18\x01 \x01(\x0b\x32\x1a.fumarole.JoinControlPlaneH\x00\x12/\n\rcommit_offset\x18\x02 \x01(\x0b\x32\x16.fumarole.CommitOffsetH\x00\x12\x34\n\tpoll_hist\x18\x03 \x01(\x0b\x32\x1f.fumarole.PollBlockchainHistoryH\x00\x12\x1e\n\x04ping\x18\x04 \x01(\x0b\x32\x0e.fumarole.PingH\x00\x42\t\n\x07\x63ommand\"\xdb\x01\n\x0f\x43ontrolResponse\x12\x33\n\x04init\x18\x01 \x01(\x0b\x32#.fumarole.InitialConsumerGroupStateH\x00\x12\x35\n\rcommit_offset\x18\x02 \x01(\x0b\x32\x1c.fumarole.CommitOffsetResultH\x00\x12\x30\n\tpoll_hist\x18\x03 \x01(\x0b\x32\x1b.fumarole.BlockchainHistoryH\x00\x12\x1e\n\x04pong\x18\x04 \x01(\x0b\x32\x0e.fumarole.PongH\x00\x42\n\n\x08response\"6\n\x12\x43ommitOffsetResult\x12\x0e\n\x06offset\x18\x01 \x01(\x03\x12\x10\n\x08shard_id\x18\x02 \x01(\x05\"\xce\x01\n\x19InitialConsumerGroupState\x12\x15\n\rblockchain_id\x18\x01 \x01(\x0c\x12]\n\x16last_committed_offsets\x18\x02 \x03(\x0b\x32=.fumarole.InitialConsumerGroupState.LastCommittedOffsetsEntry\x1a;\n\x19LastCommittedOffsetsEntry\x12\x0b\n\x03key\x18\x01 \x01(\x05\x12\r\n\x05value\x18\x02 \x01(\x03:\x02\x38\x01\"8\n\x1b\x43reateConsumerGroupResponse\x12\x19\n\x11\x63onsumer_group_id\x18\x01 \x01(\t\"\x9d\x01\n\x1a\x43reateConsumerGroupRequest\x12\x1b\n\x13\x63onsumer_group_name\x18\x01 \x01(\t\x12<\n\x15initial_offset_policy\x18\x02 \x01(\x0e\x32\x1d.fumarole.InitialOffsetPolicy\x12\x16\n\tfrom_slot\x18\x03 \x01(\x04H\x00\x88\x01\x01\x42\x0c\n\n_from_slot*0\n\x13InitialOffsetPolicy\x12\n\n\x06LATEST\x10\x00\x12\r\n\tFROM_SLOT\x10\x01\x32\xd1\x06\n\x08\x46umarole\x12\\\n\x14GetConsumerGroupInfo\x12%.fumarole.GetConsumerGroupInfoRequest\x1a\x1b.fumarole.ConsumerGroupInfo\"\x00\x12\x61\n\x12ListConsumerGroups\x12#.fumarole.ListConsumerGroupsRequest\x1a$.fumarole.ListConsumerGroupsResponse\"\x00\x12\x64\n\x13\x44\x65leteConsumerGroup\x12$.fumarole.DeleteConsumerGroupRequest\x1a%.fumarole.DeleteConsumerGroupResponse\"\x00\x12\x64\n\x13\x43reateConsumerGroup\x12$.fumarole.CreateConsumerGroupRequest\x1a%.fumarole.CreateConsumerGroupResponse\"\x00\x12I\n\rDownloadBlock\x12\x1c.fumarole.DownloadBlockShard\x1a\x16.fumarole.DataResponse\"\x00\x30\x01\x12\x44\n\rSubscribeData\x12\x15.fumarole.DataCommand\x1a\x16.fumarole.DataResponse\"\x00(\x01\x30\x01\x12L\n\x0bGetChainTip\x12\x1c.fumarole.GetChainTipRequest\x1a\x1d.fumarole.GetChainTipResponse\"\x00\x12\x46\n\tSubscribe\x12\x18.fumarole.ControlCommand\x1a\x19.fumarole.ControlResponse\"\x00(\x01\x30\x01\x12@\n\x07Version\x12\x18.fumarole.VersionRequest\x1a\x19.fumarole.VersionResponse\"\x00\x12O\n\x0cGetSlotRange\x12\x1d.fumarole.GetSlotRangeRequest\x1a\x1e.fumarole.GetSlotRangeResponse\"\x00P\x00\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fumarole_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_GETCHAINTIPRESPONSE_SHARDTOMAXOFFSETMAPENTRY']._loaded_options = None
  _globals['_GETCHAINTIPRESPONSE_SHARDTOMAXOFFSETMAPENTRY']._serialized_options = b'8\001'
  _globals['_BLOCKFILTERS_ACCOUNTSENTRY']._loaded_options = None
  _globals['_BLOCKFILTERS_ACCOUNTSENTRY']._serialized_options = b'8\001'
  _globals['_BLOCKFILTERS_TRANSACTIONSENTRY']._loaded_options = None
  _globals['_BLOCKFILTERS_TRANSACTIONSENTRY']._serialized_options = b'8\001'
  _globals['_BLOCKFILTERS_ENTRIESENTRY']._loaded_options = None
  _globals['_BLOCKFILTERS_ENTRIESENTRY']._serialized_options = b'8\001'
  _globals['_BLOCKFILTERS_BLOCKSMETAENTRY']._loaded_options = None
  _globals['_BLOCKFILTERS_BLOCKSMETAENTRY']._serialized_options = b'8\001'
  _globals['_INITIALCONSUMERGROUPSTATE_LASTCOMMITTEDOFFSETSENTRY']._loaded_options = None
  _globals['_INITIALCONSUMERGROUPSTATE_LASTCOMMITTEDOFFSETSENTRY']._serialized_options = b'8\001'
  _globals['_INITIALOFFSETPOLICY']._serialized_start=3669
  _globals['_INITIALOFFSETPOLICY']._serialized_end=3717
  _globals['_GETSLOTRANGEREQUEST']._serialized_start=42
  _globals['_GETSLOTRANGEREQUEST']._serialized_end=86
  _globals['_GETSLOTRANGERESPONSE']._serialized_start=88
  _globals['_GETSLOTRANGERESPONSE']._serialized_end=169
  _globals['_GETCHAINTIPREQUEST']._serialized_start=171
  _globals['_GETCHAINTIPREQUEST']._serialized_end=214
  _globals['_GETCHAINTIPRESPONSE']._serialized_start=217
  _globals['_GETCHAINTIPRESPONSE']._serialized_end=410
  _globals['_GETCHAINTIPRESPONSE_SHARDTOMAXOFFSETMAPENTRY']._serialized_start=352
  _globals['_GETCHAINTIPRESPONSE_SHARDTOMAXOFFSETMAPENTRY']._serialized_end=410
  _globals['_VERSIONREQUEST']._serialized_start=412
  _globals['_VERSIONREQUEST']._serialized_end=428
  _globals['_VERSIONRESPONSE']._serialized_start=430
  _globals['_VERSIONRESPONSE']._serialized_end=464
  _globals['_GETCONSUMERGROUPINFOREQUEST']._serialized_start=466
  _globals['_GETCONSUMERGROUPINFOREQUEST']._serialized_end=524
  _globals['_DELETECONSUMERGROUPREQUEST']._serialized_start=526
  _globals['_DELETECONSUMERGROUPREQUEST']._serialized_end=583
  _globals['_DELETECONSUMERGROUPRESPONSE']._serialized_start=585
  _globals['_DELETECONSUMERGROUPRESPONSE']._serialized_end=631
  _globals['_LISTCONSUMERGROUPSREQUEST']._serialized_start=633
  _globals['_LISTCONSUMERGROUPSREQUEST']._serialized_end=660
  _globals['_LISTCONSUMERGROUPSRESPONSE']._serialized_start=662
  _globals['_LISTCONSUMERGROUPSRESPONSE']._serialized_end=744
  _globals['_CONSUMERGROUPINFO']._serialized_start=746
  _globals['_CONSUMERGROUPINFO']._serialized_end=847
  _globals['_GETSLOTLAGINFOREQUEST']._serialized_start=849
  _globals['_GETSLOTLAGINFOREQUEST']._serialized_end=901
  _globals['_BLOCKFILTERS']._serialized_start=904
  _globals['_BLOCKFILTERS']._serialized_end=1517
  _globals['_BLOCKFILTERS_ACCOUNTSENTRY']._serialized_start=1155
  _globals['_BLOCKFILTERS_ACCOUNTSENTRY']._serialized_end=1242
  _globals['_BLOCKFILTERS_TRANSACTIONSENTRY']._serialized_start=1244
  _globals['_BLOCKFILTERS_TRANSACTIONSENTRY']._serialized_end=1339
  _globals['_BLOCKFILTERS_ENTRIESENTRY']._serialized_start=1341
  _globals['_BLOCKFILTERS_ENTRIESENTRY']._serialized_end=1424
  _globals['_BLOCKFILTERS_BLOCKSMETAENTRY']._serialized_start=1426
  _globals['_BLOCKFILTERS_BLOCKSMETAENTRY']._serialized_end=1517
  _globals['_DOWNLOADBLOCKSHARD']._serialized_start=1520
  _globals['_DOWNLOADBLOCKSHARD']._serialized_end=1669
  _globals['_PING']._serialized_start=1671
  _globals['_PING']._serialized_end=1694
  _globals['_PONG']._serialized_start=1696
  _globals['_PONG']._serialized_end=1719
  _globals['_DATACOMMAND']._serialized_start=1722
  _globals['_DATACOMMAND']._serialized_end=1857
  _globals['_BLOCKSHARDDOWNLOADFINISH']._serialized_start=1859
  _globals['_BLOCKSHARDDOWNLOADFINISH']._serialized_end=1885
  _globals['_BLOCKNOTFOUND']._serialized_start=1887
  _globals['_BLOCKNOTFOUND']._serialized_end=1963
  _globals['_DATAERROR']._serialized_start=1965
  _globals['_DATAERROR']._serialized_end=2031
  _globals['_DATARESPONSE']._serialized_start=2034
  _globals['_DATARESPONSE']._serialized_end=2178
  _globals['_COMMITOFFSET']._serialized_start=2180
  _globals['_COMMITOFFSET']._serialized_end=2228
  _globals['_POLLBLOCKCHAINHISTORY']._serialized_start=2230
  _globals['_POLLBLOCKCHAINHISTORY']._serialized_end=2329
  _globals['_BLOCKCHAINEVENT']._serialized_start=2332
  _globals['_BLOCKCHAINEVENT']._serialized_end=2603
  _globals['_BLOCKCHAINHISTORY']._serialized_start=2605
  _globals['_BLOCKCHAINHISTORY']._serialized_end=2667
  _globals['_JOINCONTROLPLANE']._serialized_start=2669
  _globals['_JOINCONTROLPLANE']._serialized_end=2745
  _globals['_CONTROLCOMMAND']._serialized_start=2748
  _globals['_CONTROLCOMMAND']._serialized_end=2962
  _globals['_CONTROLRESPONSE']._serialized_start=2965
  _globals['_CONTROLRESPONSE']._serialized_end=3184
  _globals['_COMMITOFFSETRESULT']._serialized_start=3186
  _globals['_COMMITOFFSETRESULT']._serialized_end=3240
  _globals['_INITIALCONSUMERGROUPSTATE']._serialized_start=3243
  _globals['_INITIALCONSUMERGROUPSTATE']._serialized_end=3449
  _globals['_INITIALCONSUMERGROUPSTATE_LASTCOMMITTEDOFFSETSENTRY']._serialized_start=3390
  _globals['_INITIALCONSUMERGROUPSTATE_LASTCOMMITTEDOFFSETSENTRY']._serialized_end=3449
  _globals['_CREATECONSUMERGROUPRESPONSE']._serialized_start=3451
  _globals['_CREATECONSUMERGROUPRESPONSE']._serialized_end=3507
  _globals['_CREATECONSUMERGROUPREQUEST']._serialized_start=3510
  _globals['_CREATECONSUMERGROUPREQUEST']._serialized_end=3667
  _globals['_FUMAROLE']._serialized_start=3720
  _globals['_FUMAROLE']._serialized_end=4569
# @@protoc_insertion_point(module_scope)



import logging
from typing import Optional
import grpc
from yellowstone_fumarole_client.config import FumaroleConfig
from yellowstone_fumarole_proto.fumarole_pb2_grpc import FumaroleStub

X_TOKEN_HEADER = "x-token"


def _triton_sign_request(
    callback: grpc.AuthMetadataPluginCallback,
    x_token: Optional[str],
    error: Optional[Exception],
):
    # WARNING: metadata is a 1d-tuple (<value>,), the last comma is necessary
    metadata = ((X_TOKEN_HEADER, x_token),)
    return callback(metadata, error)


class TritonAuthMetadataPlugin(grpc.AuthMetadataPlugin):
    """Metadata wrapper for raw access token credentials."""

    def __init__(self, x_token: str):
        self.x_token = x_token

    def __call__(
        self,
        context: grpc.AuthMetadataContext,
        callback: grpc.AuthMetadataPluginCallback,
    ):
        return _triton_sign_request(callback, self.x_token, None)


# Because of a bug in grpcio library, multiple inheritance of ClientInterceptor subclasses does not work.
# You have to create a new class for each type of interceptor you want to use.
class MetadataInterceptor(
    grpc.aio.UnaryStreamClientInterceptor,
    grpc.aio.StreamUnaryClientInterceptor,
    grpc.aio.StreamStreamClientInterceptor,
    grpc.aio.UnaryUnaryClientInterceptor,
):

    def __init__(self, metadata):
        if isinstance(metadata, dict):
            metadata = metadata.items()
        self.metadata = list(metadata)

    async def intercept_unary_unary(
        self, continuation, client_call_details: grpc.aio.ClientCallDetails, request
    ):
        # logging.debug("intercept_unary_unary")
        new_details = client_call_details._replace(
            metadata=self._merge_metadata(client_call_details.metadata)
        )
        return await continuation(new_details, request)

    async def intercept_unary_stream(
        self, continuation, client_call_details: grpc.aio.ClientCallDetails, request
    ):
        # logging.debug("intercept_unary_stream")
        new_details = client_call_details._replace(
            metadata=self._merge_metadata(client_call_details.metadata)
        )
        return await continuation(new_details, request)

    async def intercept_stream_unary(
        self, continuation, client_call_details: grpc.aio.ClientCallDetails, request
    ):
        # logging.debug("intercept_stream_unary")
        new_details = client_call_details._replace(
            metadata=self._merge_metadata(client_call_details.metadata)
        )
        return await continuation(new_details, request)

    async def intercept_stream_stream(
        self, continuation, client_call_details: grpc.aio.ClientCallDetails, request
    ):
        # logging.debug("intercept_stream_stream")
        new_details = client_call_details._replace(
            metadata=self._merge_metadata(client_call_details.metadata)
        )
        return await continuation(new_details, request)

    def unary_stream_interceptor(self) -> grpc.aio.UnaryStreamClientInterceptor:
        this = self

        class Interceptor(grpc.aio.UnaryStreamClientInterceptor):
            async def intercept_unary_stream(self, *args):
                return await this.intercept_unary_stream(*args)

        return Interceptor()

    def stream_unary_interceptor(self) -> grpc.aio.StreamUnaryClientInterceptor:
        this = self

        class Interceptor(grpc.aio.StreamUnaryClientInterceptor):
            async def intercept_stream_unary(self, *args):
                return await this.intercept_stream_unary(*args)

        return Interceptor()

    def stream_stream_interceptor(self) -> grpc.aio.StreamStreamClientInterceptor:
        this = self

        class Interceptor(grpc.aio.StreamStreamClientInterceptor):
            async def intercept_stream_stream(self, *args):
                return await this.intercept_stream_stream(*args)

        return Interceptor()

    def unary_unary_interceptor(self) -> grpc.aio.UnaryUnaryClientInterceptor:
        this = self

        class Interceptor(grpc.aio.UnaryUnaryClientInterceptor):
            async def intercept_unary_unary(self, *args):
                return await this.intercept_unary_unary(*args)

        return Interceptor()

    def interceptors(self) -> list[grpc.aio.ClientInterceptor]:
        return [
            self.unary_unary_interceptor(),
            self.unary_stream_interceptor(),
            self.stream_unary_interceptor(),
            self.stream_stream_interceptor(),
        ]

    def _merge_metadata(self, existing):
        result = list(existing or []) + self.metadata
        return result


class FumaroleGrpcConnector:
    logger = logging.getLogger(__name__)

    def __init__(self, config: FumaroleConfig, endpoint: str):
        self.config = config
        self.endpoint = endpoint

    async def connect(self, *grpc_options) -> FumaroleStub:
        options = [("grpc.max_receive_message_length", 111111110), *grpc_options]
        interceptors = MetadataInterceptor(self.config.x_metadata).interceptors()
        compression = (
            grpc.Compression.Gzip
            if self.config.response_compression == "gzip"
            else None
        )
        if self.config.x_token is not None:
            auth = TritonAuthMetadataPlugin(self.config.x_token)
            # ssl_creds allow you to use our https endpoint
            # grpc.ssl_channel_credentials with no arguments will look through your CA trust store.
            ssl_creds = grpc.ssl_channel_credentials()

            # call credentials will be sent on each request if setup with composite_channel_credentials.
            call_creds: grpc.CallCredentials = grpc.metadata_call_credentials(auth)

            # Combined creds will store the channel creds aswell as the call credentials
            combined_creds = grpc.composite_channel_credentials(ssl_creds, call_creds)
            FumaroleGrpcConnector.logger.debug(
                "Using secure channel with x-token authentication"
            )
            channel = grpc.aio.secure_channel(
                self.endpoint,
                credentials=combined_creds,
                options=options,
                compression=compression,
                interceptors=interceptors,
            )
        else:
            FumaroleGrpcConnector.logger.debug(
                "Using insecure channel without authentication"
            )
            channel = grpc.aio.insecure_channel(
                self.endpoint,
                options=options,
                interceptors=interceptors,
                compression=compression,
            )

        return FumaroleStub(channel)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: solana-storage.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'solana-storage.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x14solana-storage.proto\x12\x1dsolana.storage.ConfirmedBlock\"\xa1\x03\n\x0e\x43onfirmedBlock\x12\x1a\n\x12previous_blockhash\x18\x01 \x01(\t\x12\x11\n\tblockhash\x18\x02 \x01(\t\x12\x13\n\x0bparent_slot\x18\x03 \x01(\x04\x12I\n\x0ctransactions\x18\x04 \x03(\x0b\x32\x33.solana.storage.ConfirmedBlock.ConfirmedTransaction\x12\x36\n\x07rewards\x18\x05 \x03(\x0b\x32%.solana.storage.ConfirmedBlock.Reward\x12@\n\nblock_time\x18\x06 \x01(\x0b\x32,.solana.storage.ConfirmedBlock.UnixTimestamp\x12@\n\x0c\x62lock_height\x18\x07 \x01(\x0b\x32*.solana.storage.ConfirmedBlock.BlockHeight\x12\x44\n\x0enum_partitions\x18\x08 \x01(\x0b\x32,.solana.storage.ConfirmedBlock.NumPartitions\"\x9b\x01\n\x14\x43onfirmedTransaction\x12?\n\x0btransaction\x18\x01 \x01(\x0b\x32*.solana.storage.ConfirmedBlock.Transaction\x12\x42\n\x04meta\x18\x02 \x01(\x0b\x32\x34.solana.storage.ConfirmedBlock.TransactionStatusMeta\"Z\n\x0bTransaction\x12\x12\n\nsignatures\x18\x01 \x03(\x0c\x12\x37\n\x07message\x18\x02 \x01(\x0b\x32&.solana.storage.ConfirmedBlock.Message\"\xad\x02\n\x07Message\x12<\n\x06header\x18\x01 \x01(\x0b\x32,.solana.storage.ConfirmedBlock.MessageHeader\x12\x14\n\x0c\x61\x63\x63ount_keys\x18\x02 \x03(\x0c\x12\x18\n\x10recent_blockhash\x18\x03 \x01(\x0c\x12H\n\x0cinstructions\x18\x04 \x03(\x0b\x32\x32.solana.storage.ConfirmedBlock.CompiledInstruction\x12\x11\n\tversioned\x18\x05 \x01(\x08\x12W\n\x15\x61\x64\x64ress_table_lookups\x18\x06 \x03(\x0b\x32\x38.solana.storage.ConfirmedBlock.MessageAddressTableLookup\"~\n\rMessageHeader\x12\x1f\n\x17num_required_signatures\x18\x01 \x01(\r\x12$\n\x1cnum_readonly_signed_accounts\x18\x02 \x01(\r\x12&\n\x1enum_readonly_unsigned_accounts\x18\x03 \x01(\r\"d\n\x19MessageAddressTableLookup\x12\x13\n\x0b\x61\x63\x63ount_key\x18\x01 \x01(\x0c\x12\x18\n\x10writable_indexes\x18\x02 \x01(\x0c\x12\x18\n\x10readonly_indexes\x18\x03 \x01(\x0c\"\x82\x06\n\x15TransactionStatusMeta\x12<\n\x03\x65rr\x18\x01 \x01(\x0b\x32/.solana.storage.ConfirmedBlock.TransactionError\x12\x0b\n\x03\x66\x65\x65\x18\x02 \x01(\x04\x12\x14\n\x0cpre_balances\x18\x03 \x03(\x04\x12\x15\n\rpost_balances\x18\x04 \x03(\x04\x12L\n\x12inner_instructions\x18\x05 \x03(\x0b\x32\x30.solana.storage.ConfirmedBlock.InnerInstructions\x12\x1f\n\x17inner_instructions_none\x18\n \x01(\x08\x12\x14\n\x0clog_messages\x18\x06 \x03(\t\x12\x19\n\x11log_messages_none\x18\x0b \x01(\x08\x12G\n\x12pre_token_balances\x18\x07 \x03(\x0b\x32+.solana.storage.ConfirmedBlock.TokenBalance\x12H\n\x13post_token_balances\x18\x08 \x03(\x0b\x32+.solana.storage.ConfirmedBlock.TokenBalance\x12\x36\n\x07rewards\x18\t \x03(\x0b\x32%.solana.storage.ConfirmedBlock.Reward\x12!\n\x19loaded_writable_addresses\x18\x0c \x03(\x0c\x12!\n\x19loaded_readonly_addresses\x18\r \x03(\x0c\x12>\n\x0breturn_data\x18\x0e \x01(\x0b\x32).solana.storage.ConfirmedBlock.ReturnData\x12\x18\n\x10return_data_none\x18\x0f \x01(\x08\x12#\n\x16\x63ompute_units_consumed\x18\x10 \x01(\x04H\x00\x88\x01\x01\x12\x17\n\ncost_units\x18\x11 \x01(\x04H\x01\x88\x01\x01\x42\x19\n\x17_compute_units_consumedB\r\n\x0b_cost_units\"\x1f\n\x10TransactionError\x12\x0b\n\x03\x65rr\x18\x01 \x01(\x0c\"i\n\x11InnerInstructions\x12\r\n\x05index\x18\x01 \x01(\r\x12\x45\n\x0cinstructions\x18\x02 \x03(\x0b\x32/.solana.storage.ConfirmedBlock.InnerInstruction\"x\n\x10InnerInstruction\x12\x18\n\x10program_id_index\x18\x01 \x01(\r\x12\x10\n\x08\x61\x63\x63ounts\x18\x02 \x01(\x0c\x12\x0c\n\x04\x64\x61ta\x18\x03 \x01(\x0c\x12\x19\n\x0cstack_height\x18\x04 \x01(\rH\x00\x88\x01\x01\x42\x0f\n\r_stack_height\"O\n\x13\x43ompiledInstruction\x12\x18\n\x10program_id_index\x18\x01 \x01(\r\x12\x10\n\x08\x61\x63\x63ounts\x18\x02 \x01(\x0c\x12\x0c\n\x04\x64\x61ta\x18\x03 \x01(\x0c\"\x9d\x01\n\x0cTokenBalance\x12\x15\n\raccount_index\x18\x01 \x01(\r\x12\x0c\n\x04mint\x18\x02 \x01(\t\x12\x45\n\x0fui_token_amount\x18\x03 \x01(\x0b\x32,.solana.storage.ConfirmedBlock.UiTokenAmount\x12\r\n\x05owner\x18\x04 \x01(\t\x12\x12\n\nprogram_id\x18\x05 \x01(\t\"^\n\rUiTokenAmount\x12\x11\n\tui_amount\x18\x01 \x01(\x01\x12\x10\n\x08\x64\x65\x63imals\x18\x02 \x01(\r\x12\x0e\n\x06\x61mount\x18\x03 \x01(\t\x12\x18\n\x10ui_amount_string\x18\x04 \x01(\t\".\n\nReturnData\x12\x12\n\nprogram_id\x18\x01 \x01(\x0c\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\x0c\"\x94\x01\n\x06Reward\x12\x0e\n\x06pubkey\x18\x01 \x01(\t\x12\x10\n\x08lamports\x18\x02 \x01(\x03\x12\x14\n\x0cpost_balance\x18\x03 \x01(\x04\x12>\n\x0breward_type\x18\x04 \x01(\x0e\x32).solana.storage.ConfirmedBlock.RewardType\x12\x12\n\ncommission\x18\x05 \x01(\t\"\x87\x01\n\x07Rewards\x12\x36\n\x07rewards\x18\x01 \x03(\x0b\x32%.solana.storage.ConfirmedBlock.Reward\x12\x44\n\x0enum_partitions\x18\x02 \x01(\x0b\x32,.solana.storage.ConfirmedBlock.NumPartitions\"\"\n\rUnixTimestamp\x12\x11\n\ttimestamp\x18\x01 \x01(\x03\"#\n\x0b\x42lockHeight\x12\x14\n\x0c\x62lock_height\x18\x01 \x01(\x04\"\'\n\rNumPartitions\x12\x16\n\x0enum_partitions\x18\x01 \x01(\x04*I\n\nRewardType\x12\x0f\n\x0bUnspecified\x10\x00\x12\x07\n\x03\x46\x65\x65\x10\x01\x12\x08\n\x04Rent\x10\x02\x12\x0b\n\x07Staking\x10\x03\x12\n\n\x06Voting\x10\x04\x42;Z9github.com/rpcpool/yellowstone-grpc/examples/golang/protob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'solana_storage_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z9github.com/rpcpool/yellowstone-grpc/examples/golang/proto'
  _globals['_REWARDTYPE']._serialized_start=3082
  _globals['_REWARDTYPE']._serialized_end=3155
  _globals['_CONFIRMEDBLOCK']._serialized_start=56
  _globals['_CONFIRMEDBLOCK']._serialized_end=473
  _globals['_CONFIRMEDTRANSACTION']._serialized_start=476
  _globals['_CONFIRMEDTRANSACTION']._serialized_end=631
  _globals['_TRANSACTION']._serialized_start=633
  _globals['_TRANSACTION']._serialized_end=723
  _globals['_MESSAGE']._serialized_start=726
  _globals['_MESSAGE']._serialized_end=1027
  _globals['_MESSAGEHEADER']._serialized_start=1029
  _globals['_MESSAGEHEADER']._serialized_end=1155
  _globals['_MESSAGEADDRESSTABLELOOKUP']._serialized_start=1157
  _globals['_MESSAGEADDRESSTABLELOOKUP']._serialized_end=1257
  _globals['_TRANSACTIONSTATUSMETA']._serialized_start=1260
  _globals['_TRANSACTIONSTATUSMETA']._serialized_end=2030
  _globals['_TRANSACTIONERROR']._serialized_start=2032
  _globals['_TRANSACTIONERROR']._serialized_end=2063
  _globals['_INNERINSTRUCTIONS']._serialized_start=2065
  _globals['_INNERINSTRUCTIONS']._serialized_end=2170
  _globals['_INNERINSTRUCTION']._serialized_start=2172
  _globals['_INNERINSTRUCTION']._serialized_end=2292
  _globals['_COMPILEDINSTRUCTION']._serialized_start=2294
  _globals['_COMPILEDINSTRUCTION']._serialized_end=2373
  _globals['_TOKENBALANCE']._serialized_start=2376
  _globals['_TOKENBALANCE']._serialized_end=2533
  _globals['_UITOKENAMOUNT']._serialized_start=2535
  _globals['_UITOKENAMOUNT']._serialized_end=2629
  _globals['_RETURNDATA']._serialized_start=2631
  _globals['_RETURNDATA']._serialized_end=2677
  _globals['_REWARD']._serialized_start=2680
  _globals['_REWARD']._serialized_end=2828
  _globals['_REWARDS']._serialized_start=2831
  _globals['_REWARDS']._serialized_end=2966
  _globals['_UNIXTIMESTAMP']._serialized_start=2968
  _globals['_UNIXTIMESTAMP']._serialized_end=3002
  _globals['_BLOCKHEIGHT']._serialized_start=3004
  _globals['_BLOCKHEIGHT']._serialized_end=3039
  _globals['_NUMPARTITIONS']._serialized_start=3041
  _globals['_NUMPARTITIONS']._serialized_end=3080
# @@protoc_insertion_point(module_scope)



from dataclasses import dataclass
from typing import Dict, Literal, Optional
import yaml


SUPPORTED_COMPRESSION = ["gzip"]
SupportedCompression = Literal["gzip"]


@dataclass
class FumaroleConfig:
    endpoint: str
    x_token: Optional[str] = None
    max_decoding_message_size_bytes: int = 512_000_000
    x_metadata: Dict[str, str] = None
    response_compression: Optional[SupportedCompression] = None

    def __post_init__(self):
        self.x_metadata = self.x_metadata or {}

    @classmethod
    def from_yaml(cls, fileobj) -> "FumaroleConfig":
        data = yaml.safe_load(fileobj)
        response_compression = data.get(
            "response_compression", cls.response_compression
        )
        if (
            response_compression is not None
            and response_compression not in SUPPORTED_COMPRESSION
        ):
            raise ValueError(f"response_compression must be in {SUPPORTED_COMPRESSION}")
        return cls(
            endpoint=data["endpoint"],
            x_token=data.get("x-token") or data.get("x_token"),
            max_decoding_message_size_bytes=data.get(
                "max_decoding_message_size_bytes", cls.max_decoding_message_size_bytes
            ),
            x_metadata=data.get("x-metadata", {}),
            response_compression=data.get(
                "response_compression", cls.response_compression
            ),
        )
